'--------------------------------------------------------------------------------
'  NZTV2.vbs (v2.1) 
'--------------------------------------------------------------------------------
'
' This script rips information from http://tvnz.co.nz/view-preempt/tvnz_epg_skin/
' to work out what is current showing on TV and what is coming up next.
'
' This script currently only works with five free to air NZ channels:
'      - TV One
'      - TV2
'      - TV3
'      - TV4
'      - Prime
'
'
' NOTES:
'
' - Use the the "generateListing" function to update your TV listings. This
'   create a .tmp file in your Samurize directory that contains the raw TV
'   listings.
'
' - The script can then be used in 2 different ways.
'
' 	1) Use the "generateTextListing" function. This writes a file
'	   called TVlistings.txt to your Samurize directory. This file is
'	   in the following format:
'
'		<TV channel name>
'		<current show start time>
'		<current show end time>
'		<current show title>
'		<current show % completion>
'		<next show start time>
'		<next show end time>
'		<next show title>
'		<TV channel name>
'		<current show start time>
'		.
'		.
'		.
'
'	   Just use TextFile meters to show the stuff you want in Samurize.
'	   This way works best cuz the listings update on your desktop
'	   whenever the file contents change. I recommend this method.
'	   The sample config file included with this script uses this method.
'	   
'
' 	2) Use the appropriate functions to display what you want in
'	   Samurize:
'
'	   getTVXNow	- gives the name of the show currently on channel X
'
'	   getTVXNext	- gives the name of the show next on channel X
'
'	   getStartTime - gives the start time of the current show (takes the
' 			  channel number - use 5 for Prime)
'
'	   getEndTime 	- gives the end time of the current show (takes the
' 			  channel number - use 5 for Prime)
'
'	   getNextStartTime - gives the start time of the next show (takes the
' 			      channel number - use 5 for Prime)
'
'	   getNextStartTime - gives the end time of the next show (takes the
' 			      channel number - use 5 for Prime)
'
'	   getPercentComplete - gives the percentage completion of the current
'				show - useful for progressbars (takes the
' 			        channel number - use 5 for Prime)
'
'
' - I recommend running the "generateListing" function no more than once every
'   five minutes (the listings are actually good for up to a day but at midnight
'   you have to refresh them). The other functions can be run as often as you 
'   like - they just fetch info from the temporary files. You shouldn't need to 
'   do this more than once every few minutes though.
'
' Version history:
'
' v2.1: Fixed to use new URL/layout
'
' v2.0: Completely rewritten to use a better and more reliable site.
'
'				                - NeM (nem@paradise.net.nz)
'------------------------------------------------------------------------------

' Adjust these paths if you want.
FILE_NAME = "C:\Program Files\Samurize\NZTV.tmp"
FILE_NAME_NEXT = "C:\Program Files\Samurize\NZTV-next.tmp"
LISTING_FILE = "C:\Program Files\Samurize\TVlistings.txt"

' Display when nothing is on/listed
NOT_AVAILABLE = "00:00" & vbCrLf & "00:00" & vbCrLf & "N/A"


FEED = vbCrlf


Function generateTextListing()
	
	textToWrite = "TV1" & FEED & getNowListing(1) & getNextListing(1)
	textToWrite = textToWrite & "TV2" & FEED & getNowListing(2) & getNextListing(2)
	textToWrite = textToWrite & "TV3" & FEED & getNowListing(3) & getNextListing(3)
	textToWrite = textToWrite & "TV4" & FEED & getNowListing(4) & getNextListing(4)
	textToWrite = textToWrite & "Prime" & FEED & getNowListing(5) & getNextListing(5)

	set fs=CreateObject("Scripting.FileSystemObject")
	set f=fs.CreateTextFile(LISTING_FILE,true)
	f.write(textToWrite)
	f.close
	set f=nothing
	set fs=nothing

End Function

Function getTV1now()
	getTV1now=getNowListing(1)
	getTV1now=Split(getTV1Now, vbCrLf)(2)
End Function

Function getTV2now()
	getTV2now=getNowListing(2)
	getTV2now=Split(getTV2Now, vbCrLf)(2)
End Function

Function getTV3now()
	getTV3now=getNowListing(3)
	getTV3now=Split(getTV3Now, vbCrLf)(2)
End Function

Function getTV4now()
	getTV4now=getNowListing(4)
	getTV4now=Split(getTV4Now, vbCrLf)(2)
End Function

Function getPrimenow()
	getPrimenow=getNowListing(5)
	getPrimenow=Split(getPrimeNow, vbCrLf)(2)
End Function

Function getTV1next()
	getTV1next=getNextListing(1)
	getTV1next=Split(getTV1Next, vbCrLf)(2)
End Function

Function getTV2next()
	getTV2next=getNextListing(2)
	getTV2next=Split(getTV2Next, vbCrLf)(2)
End Function

Function getTV3next()
	getTV3next=getNextListing(3)
	getTV3next=Split(getTV3Next, vbCrLf)(2)
End Function

Function getTV4next()
	getTV4next=getNextListing(4)
	getTV4next=Split(getTV4Next, vbCrLf)(2)
End Function

Function getPrimenext()
	getPrimenext=getNextListing(5)
	getPrimenext=Split(getPrimeNext, vbCrLf)(2)
End Function

Function getStartTime(Channel)
	getStartTime = Split(getNowListing(Channel), vbCrLf)(0)
End Function

Function getEndTime(Channel)
	getEndTime = Split(getNowListing(Channel), vbCrLf)(1)
End Function

Function getPercentComplete(Channel)
	getPercentComplete = Split(getNowListing(Channel), vbCrLf)(3)
End Function

Function getNextStartTime(Channel)
	getNextStartTime = Split(getNextListing(Channel), vbCrLf)(0)
End Function

Function getNextEndTime(Channel)
	getNextEndTime = Split(getNextListing(Channel), vbCrLf)(1)
End Function

Function generateListing()

	dim fs,f,htmlResult

	htmlResult = ReturnHTML("http://tvnz.co.nz/view-preempt/tvnz_epg_skin/")

	set fs=CreateObject("Scripting.FileSystemObject")
	set f=fs.CreateTextFile(FILE_NAME,true)
	f.write(htmlResult)
	f.close
	set f=nothing
	set fs=nothing

End Function



Private Function getNowListing(Channel)

	dim htmlResult,output,strAllInfo,junk,fs,file
	
	FirstListing = True

	htmlResult = readFile(FILE_NAME)

	' first cut everything before <!--display programmes-->
	strAllInfo = Mid(htmlResult, InStr(htmlResult, "<!--display programmes-->"))


	' jump to before correct channel
	For i=1 to Channel
		strAllInfo = Mid(strAllInfo, InStr(strAllInfo, "<table border=""0"" cellspacing=""0"" cellpadding=""0"" class=""epg_border"">")+Len("<table border=""0"" cellspacing=""0"" cellpadding=""0"" class=""epg_border"">"), Len(strAllInfo))
	Next

	TableEnd = InStr(strAllInfo, "</table>")

	endPos = 1


	While endPos < TableEnd
	
		startPos = InStr(endPos, strAllInfo, "<td class=""epg_cell_")
		
		if startpos < 1 then
			getNowListing = NOT_AVAILABLE & FEED & "0" & FEED
 			exit function
		End if

		
		endPos =  InStr(startPos, strAllInfo, "</td>")
		tmpEndPos = endPos
		if (InStr(tmpendPos+1, strAllInfo, "</td>") > 0) and (InStr(tmpendPos+1, strAllInfo, "</td>") < tableend) then
		    tmpendPos =  InStr(tmpendPos+1, strAllInfo, "</td>") ' again so that we have 2 shows
		end if


		' check if no show listed
		if (tmpendpos < 1) or (tmpendpos-startpos < 0) then 
			getNextListing = NOT_AVAILABLE & FEED & "0" & FEED
			exit function
		end if

		ChannelHTML = mid(StrAllInfo, startPos, tmpEndPos - StartPos)



		Dim objRegExpr

		'Create an instance of the regexp object
		Set objRegExpr = New regexp
			
			
		' DO THE TIMES
		objRegExpr.Pattern = "[0-9]+:[0-9]+"
		objRegExpr.Global = true

			
		'Declare a variable to hold our collection of Matches
		Dim colMatches

		'Now, Execute the regular expression search
		Set colMatches = objRegExpr.Execute(ChannelHTML)


		Dim objMatch
			
		If colMatches.Count > 1 then
		    TimeInfo = colMatches.Item(0).value
		    StartHour = Mid(TimeInfo, 1, 2)
		    StartMinute = Mid(TimeInfo, InStr(TimeInfo, ":")+1, 2)
		    TimeInfo = colMatches.Item(1).value
		    EndHour = Mid(TimeInfo, 1, 2)
		    EndMinute = Mid(TimeInfo, InStr(TimeInfo, ":")+1, 2)
		else
		    ' Last item shown for the day - assume it finishes at midnight
		    TimeInfo = colMatches.Item(0).value
		    StartHour = Mid(TimeInfo, 1, 2)
		    StartMinute = Mid(TimeInfo, InStr(TimeInfo, ":")+1, 2)
		    EndHour = "24"
		    EndMinute = "00"
		end if
		
		If FirstListing and (CInt(StartHour)*60 + CInt(StartMinute) > Hour(Time)*60+Minute(Time)) then
		    ' first listing not available
		    getNowListing = NOT_AVAILABLE & FEED & "0" & FEED
		    Exit Function
		end if

		' Wraparound time fix
		If (CInt(EndHour)*60 + CInt(EndMinute)) < (CInt(StartHour)*60 + CInt(StartMinute)) Then
			EndHour = EndHour + 24
		End If
	

			
		If  (CInt(StartHour)*60 + CInt(StartMinute) <= Hour(Time)*60+Minute(Time)) And _
		    (CInt(EndHour)*60 + CInt(EndMinute) >= Hour(Time)*60+Minute(Time)) Then	

			ShowLength = (CInt(EndHour)*60 + Cint(EndMinute))  - (CInt(StartHour)*60 + Cint(StartMinute))
			ShowPos = Hour(Time)*60 + Minute(Time) - (CInt(StartHour)*60 + Cint(StartMinute))
			Percent = ShowPos / ShowLength * 100

			If CInt(EndHour) > 23 then
				EndHour = EndHour - 24
			End If

			GetNowListing = getNowListing & StartHour & ":" & StartMinute & FEED & EndHour & ":" & EndMinute & FEED

			StartTitle = InStr(ChannelHTML, "<strong class=""epg_section_heading"">")+len("<strong class=""epg_section_heading"">")
			
			' check if no show listed
			if starttitle < 1 then
				getNowListing = NOT_AVAILABLE & FEED & "0" & FEED
				exit function			
			end if	
					
			EndTitle = InStr(StartTitle, ChannelHTML, "</strong>")



			If endtitle < 1 Then
				getNowListing = NOT_AVAILABLE & FEED & "0" & FEED
 				Exit Function
			End If
			If endtitle - starttitle < 1 Then
				getNowListing = NOT_AVAILABLE & FEED & "0" & FEED
				Exit Function
			End If

			Title = Mid(ChannelHTML, StartTitle, EndTitle - StartTitle)
			
			' get the URL
			startURL = InStr(Title, "<a target=""_blank"" href=""") + Len("<a target=""_blank"" href=""")
			if startURL > Len("<a target=""_blank"" href=""") then
				endURL = InStr(startURL,Title, """") - 1
				
				progInfoURL = Mid(Title, startURL, endURL-startURL + 1)
				title = title & "%l" & progInfoURL
			
			end if
			
			Title = TrimHTML(Title)
			
			getnowlisting = getnowlisting & title & FEED & Percent & FEED
				
			Exit Function
			
		End If
		
		
		'Clean up
		Set colMatches = Nothing
		Set objRegExpr = Nothing
		
		' Keep going...
		FirstListing = False



	Wend

	getNowListing = NOT_AVAILABLE & FEED & "0" & FEED

End Function




Private Function getNextListing(Channel)

	dim htmlResult,output,strAllInfo,junk,fs,file

	htmlResult = readFile(FILE_NAME)

	' first cut everything before <!--display programmes-->
	strAllInfo = Mid(htmlResult, InStr(htmlResult, "<!--display programmes-->"))


	' jump to before correct channel
	For i=1 to Channel
		strAllInfo = Mid(strAllInfo, InStr(strAllInfo, "<table border=""0"" cellspacing=""0"" cellpadding=""0"" class=""epg_border"">")+Len("<table border=""0"" cellspacing=""0"" cellpadding=""0"" class=""epg_border"">"), Len(strAllInfo))
	Next

	TableEnd = InStr(strAllInfo, "</table>")

	endPos = 1

	While endPos < TableEnd
	
		startPos = InStr(endpos, strAllInfo, "<td class=""epg_cell_")
		
		if startpos < 1 then
			getNextListing = NOT_AVAILABLE & FEED
 			exit function
		End if
		
		endPos =  InStr(startPos, strAllInfo, "</td>")
		tmpEndPos = endPos
		if (InStr(tmpendPos+1, strAllInfo, "</td>") > 0) and (InStr(tmpendPos+1, strAllInfo, "</td>") < tableend) then
		    tmpendPos =  InStr(tmpendPos+1, strAllInfo, "</td>") ' again so that we have 2 shows
		end if


		' check if no show listed
		if (tmpendpos < 1) or (tmpendpos-startpos < 0) then 
			getNextListing = NOT_AVAILABLE & FEED
			exit function
		end if

		ChannelHTML = mid(StrAllInfo, startPos, tmpEndPos - StartPos)

		Dim objRegExpr

		'Create an instance of the regexp object
		Set objRegExpr = New regexp
			
			
		' DO THE TIMES
		objRegExpr.Pattern = "[0-9]+:[0-9]+"
		objRegExpr.Global = true

			
		'Declare a variable to hold our collection of Matches
		Dim colMatches

		'Now, Execute the regular expression search
		Set colMatches = objRegExpr.Execute(ChannelHTML)


		Dim objMatch
			
		If colMatches.Count > 1 then
		    TimeInfo = colMatches.Item(0).value
		    StartHour = Mid(TimeInfo, 1, 2)
		    StartMinute = Mid(TimeInfo, InStr(TimeInfo, ":")+1, 2)
		    TimeInfo = colMatches.Item(1).value
		    EndHour = Mid(TimeInfo, 1, 2)
		    EndMinute = Mid(TimeInfo, InStr(TimeInfo, ":")+1, 2)
		else
		    ' Last item shown for the day - assume it finishes at midnight
		    TimeInfo = colMatches.Item(0).value
		    StartHour = Mid(TimeInfo, 1, 2)
		    StartMinute = Mid(TimeInfo, InStr(TimeInfo, ":")+1, 2)
		    EndHour = "24"
		    EndMinute = "00"
		end if

		' Wraparound time fix
		If (CInt(EndHour)*60 + CInt(EndMinute)) < (CInt(StartHour)*60 + CInt(StartMinute)) Then
			EndHour = EndHour + 24
		End If
		
		
		
			
		If ( CInt(StartHour)*60 + CInt(StartMinute) > Hour(Time)*60+Minute(Time) ) Then	

			ShowLength = (CInt(EndHour)*60 + Cint(EndMinute))  - (CInt(StartHour)*60 + Cint(StartMinute))
			ShowPos = Hour(Time)*60 + Minute(Time) - (CInt(StartHour)*60 + Cint(StartMinute))
			
			IF ShowLength <> 0 then
			    Percent = ShowPos / ShowLength * 100
			Else
			    Percent = 0
			End If

			If CInt(EndHour) > 23 then
				EndHour = EndHour - 24
			End If
			If CInt(StartHour) > 23 then
				StartHour = StartHour - 24
			End If

			GetNextListing = getNextListing & StartHour & ":" & StartMinute & FEED & EndHour & ":" & EndMinute & FEED

			StartTitle = InStr(ChannelHTML, "<strong class=""epg_section_heading"">")+len("<strong class=""epg_section_heading"">")
			
			' check if no show listed
			if starttitle < 1 then
				getNextListing = NOT_AVAILABLE & FEED
				exit function			
			end if	
					
			EndTitle = InStr(StartTitle, ChannelHTML, "</strong>")



			If endtitle < 1 Then
				getNextListing = NOT_AVAILABLE & FEED
 				Exit Function
			End If
			If endtitle - starttitle < 1 Then
				getNextListing = NOT_AVAILABLE & FEED
				Exit Function
			End If

			Title = Mid(ChannelHTML, StartTitle, EndTitle - StartTitle)
			
			' get the URL
			startURL = InStr(Title, "<a target=""_blank"" href=""") + Len("<a target=""_blank"" href=""")
			if startURL > Len("<a target=""_blank"" href=""") then
				endURL = InStr(startURL,Title, """") - 1
				
				progInfoURL = Mid(Title, startURL, endURL-startURL + 1)
				title = title & "%l" & progInfoURL
			
			end if
			
			Title = TrimHTML(Title)
			
			getnextlisting = getnextlisting & title & FEED
				
			Exit Function
			
		End If
		
		
		'Clean up
		Set colMatches = Nothing
		Set objRegExpr = Nothing
		
		' Keep going...
	Wend

	' no listing found!
	getNextListing = NOT_AVAILABLE & FEED

End Function


Private Function readFile(fileName)

	dim contents

	Set fs = CreateObject ("Scripting.FileSystemObject")
        If (fs.FileExists(fileName)) Then
		Set filePath = fs.GetFile(fileName)
		set File = filePath.OpenAsTextStream (1, -2)

		contents = file.readall

		File.Close ' Closes the file
		Set filePath = nothing
		Set fs = nothing
		Set File = nothing
	Else 
		Exit Function
        End If

	' return file contents
	readFile = contents

End Function


Private Function ReturnHTML(sURL)
	Dim objXMLHTTP,HTML
	Set objXMLHTTP = CreateObject("Microsoft.XMLHTTP")
	objXMLHTTP.Open "GET", sURL, False
	objXMLHTTP.Send
	HTML = objXMLHTTP.responseBody
	Set objRS = CreateObject("ADODB.Recordset")
	objRS.Fields.Append "txt", 200, 45000, &H00000080
	objRS.Open
	objRS.AddNew
	objRS.Fields("txt").AppendChunk HTML
	ReturnHTML = objRS("txt").Value
	objRS.Close
	Set objRS = Nothing
	Set objXMLHTTP = Nothing
End Function

'very simple function that will remove all html tags
Private Function TrimHTML(str)
pos_deb = InStr(1, str, "<")
Do Until pos_deb = 0
    pos_fin = InStr(pos_deb, str, ">")
    part_d = Mid(str, 1, pos_deb - 1)
    part_f = Mid(str, pos_fin + 1, Len(str) - pos_fin)
    str = part_d & part_f
    pos_deb = InStr(1, str, "<")
Loop
TrimHTML = str
End Function
